﻿using System;
using System.Linq;
using System.Drawing;
using System.Threading.Tasks;
using System.Windows.Forms;
using System.Collections.Generic;

namespace Tech.DynamicCoding.PicGen
{
    public partial class Form1 : Form
    {
        public Form1()
        {
            InitializeComponent();
        }

        private void Form1_Load(object sender, EventArgs e)
        {
            _pictureBox.Image = new Bitmap(_pictureBox.Width, _pictureBox.Height);
            Redraw();
        }

        private void _button_Click(object sender, EventArgs e)
        {
            Redraw();
        }

        private void Redraw()
        {
            try
            {
                Cursor = Cursors.WaitCursor;
                Draw(_textBox.Text, (Bitmap)_pictureBox.Image);
                _pictureBox.Invalidate();
            }
            catch (Exception x)
            {
                MessageBox.Show(x.Message, Text);
            }
            finally
            {
                Cursor = Cursors.Default;
            }
        }

        private static void Draw(string sourceCode, Bitmap bitmap)
        {
            var code = CodeGenerator.CreateCode<double>("return " + sourceCode + ";",
                new CodeParameter("x", typeof(float)),
                new CodeParameter("y", typeof(float)));

            var min = double.MaxValue;
            var max = double.MinValue;
            var pictureSize = bitmap.Size;

            Parallel.ForEach(AllPicturePoints(pictureSize)
                .Select(p => ToPointF(p.X, p.Y, pictureSize))
                .Select(p => code.Execute(p.X, p.Y)), f => 
                {
                    min = f < min ? f : min;
                    max = max < f ? f : max;
                });

            Parallel.ForEach(AllPicturePoints(pictureSize), point =>
            {
                var p = ToPointF(point.X, point.Y, pictureSize);
                var f = (code.Execute(p.X, p.Y) - min)/(max - min);
                var r  = Colorize(f, 191);
                var gb = Colorize(f, 127);
                var color = Color.FromArgb(r, gb, gb);
                lock(bitmap)
                    bitmap.SetPixel(point.X, point.Y, color);
            });
        }


        private static PointF ToPointF(int x, int y, Size size)
        {
            return new PointF((float)x / size.Width - 0.5f, (float)y / size.Height - 0.5f);
        }


        private static int Colorize(double value, int colorComponent)
        {
            return 255 - (int) Math.Max(Math.Min(value * colorComponent, colorComponent), 0);
        }


        private static IEnumerable<Point> AllPicturePoints(Size size)
        {
            return from x in Enumerable.Range(0, size.Width)
                   from y in Enumerable.Range(0, size.Height)
                   select new Point(x, y);
        }
    }
}
// Math.Cos(100 * x *x - 100 * Math.Cos(10*y) * Math.Sin(15 * x))
// Math.Sin(50 * y * y - 50 * x * Math.Sin(5 * y))